#include "main.h"
//==============================================================
void app_main(void)
{
  gpio_reset_pin(CONFIG_BLINK_GPIO);
  gpio_set_direction(CONFIG_BLINK_GPIO, GPIO_MODE_OUTPUT);
  // Initialize NVS
  esp_err_t err = nvs_flash_init();
  if (err == ESP_ERR_NVS_NO_FREE_PAGES || err == ESP_ERR_NVS_NEW_VERSION_FOUND) {
      ESP_ERROR_CHECK(nvs_flash_erase());
      err = nvs_flash_init();
  }


  //listing all the key-value pairs of any type under specified partition and namespace
  nvs_iterator_t it = NULL;
  it = nvs_entry_find("nvs", NULL, NVS_TYPE_ANY);
  while (it != NULL) {
          nvs_entry_info_t info;
          nvs_entry_info(it, &info);
          it = nvs_entry_next(it);
          printf("key '%s', type '%02x' \n", info.key, info.type);
  };
  nvs_release_iterator(it);

  nvs_handle_t my_handle;
  // Open
  printf("\n");
  printf("Opening Non-Volatile Storage (NVS) handle... ");
  err = nvs_open("storage", NVS_READWRITE, &my_handle);
  if (err != ESP_OK) {
    printf("Error (%s) opening NVS handle!\n", esp_err_to_name(err));
  }
  else
  {
    printf("Done\n");

    // Write
    int8_t val01 = -108;
    uint8_t val02 = 198;
    int16_t val03 = -10135;
    uint16_t  val04 = 61145;
    int32_t val05 = -101459;
    uint32_t val06 = 3352582294;
    int64_t val07 = -4047512760478312334;
    uint64_t val08 = 7214561357951254753;

    printf("Updating data in NVS ... ");
    err = nvs_set_i8(my_handle, "dt01", val01);
    printf((err != ESP_OK) ? "Failed!\n" : "Done\n");
    err = nvs_set_u8(my_handle, "dt02", val02);
    printf((err != ESP_OK) ? "Failed!\n" : "Done\n");
    err = nvs_set_i16(my_handle, "dt03", val03);
    printf((err != ESP_OK) ? "Failed!\n" : "Done\n");
    err = nvs_set_u16(my_handle, "dt04", val04);
    printf((err != ESP_OK) ? "Failed!\n" : "Done\n");
    err = nvs_set_i32(my_handle, "dt05", val05);
    printf((err != ESP_OK) ? "Failed!\n" : "Done\n");
    err = nvs_set_u32(my_handle, "dt06", val06);
    printf((err != ESP_OK) ? "Failed!\n" : "Done\n");
    err = nvs_set_i64(my_handle, "dt07", val07);
    printf((err != ESP_OK) ? "Failed!\n" : "Done\n");
    err = nvs_set_u64(my_handle, "dt08", val08);
    printf((err != ESP_OK) ? "Failed!\n" : "Done\n");

    err = nvs_set_str(my_handle, "MySSID", "MyLogin");
    printf((err != ESP_OK) ? "Failed!\n" : "Done\n");
    err = nvs_set_str(my_handle, "MyPASS", "123abc7890");
    printf((err != ESP_OK) ? "Failed!\n" : "Done\n");
    printf("Committing updates in NVS ... ");
    err = nvs_commit(my_handle);

    printf((err != ESP_OK) ? "Failed!\n" : "Done\n");

    // Read
    int8_t res01 = 0;
    uint8_t res02 = 0;
    int16_t res03 = 0;
    uint16_t res04 = 0;
    int32_t res05 = 0;
    uint32_t res06 = 0;
    int64_t res07 = 0;
    uint64_t res08 = 0;

    nvs_get_i8(my_handle, "dt01", &res01);
    printf("dt01: %d\n", res01);
    nvs_get_u8(my_handle, "dt02", &res02);
    printf("dt02: %d\n", res02);
    nvs_get_i16(my_handle, "dt03", &res03);
    printf("dt03: %d\n", res03);
    nvs_get_u16(my_handle, "dt04", &res04);
    printf("dt04: %d\n", res04);
    nvs_get_i32(my_handle, "dt05", &res05);
    printf("dt05: %d\n", res05);
    nvs_get_u32(my_handle, "dt06", &res06);
    printf("dt06: %u\n", res06);
    nvs_get_i64(my_handle, "dt07", &res07);
    printf("dt07: %lld\n", res07);
    nvs_get_u64(my_handle, "dt08", &res08);
    printf("dt08: %llu\n", res08);

    printf("Reading MySSID from NVS ... ");
    size_t nvs_required_size;
    err = nvs_get_str(my_handle, "MySSID", NULL, &nvs_required_size);
    switch (err)
    {
       case ESP_OK:
           printf("Done\n");
           char *nvs_ret_data;
           nvs_ret_data = malloc(nvs_required_size);
           err = nvs_get_str(my_handle, "MySSID", nvs_ret_data, &nvs_required_size );
           printf("MySSID: %s\n", nvs_ret_data);
		   free(nvs_ret_data);
           break;
       case ESP_ERR_NVS_NOT_FOUND:
           printf("The value is not initialized yet!\n");
           break;
       default :
           printf("Error (%s) reading!\n", esp_err_to_name(err));
    }

    printf("Reading MyPASS from NVS ... ");
    err = nvs_get_str(my_handle, "MyPASS", NULL, &nvs_required_size);
    switch (err)
    {
       case ESP_OK:
           printf("Done\n");
           char *nvs_ret_data;
           nvs_ret_data = malloc(nvs_required_size);
           err = nvs_get_str(my_handle, "MyPASS", nvs_ret_data, &nvs_required_size );
           printf("MyPASS: %s\n", nvs_ret_data);
		   free(nvs_ret_data);
           break;
       case ESP_ERR_NVS_NOT_FOUND:
           printf("The value is not initialized yet!\n");
           break;
       default :
           printf("Error (%s) reading!\n", esp_err_to_name(err));
    }

    // Close
    nvs_close(my_handle);
  }

  printf("\n");

}
